#ifndef _IPNATCTL_H
#define _IPNATCTL_H
#include "NAT/ip_nat_rule.h"

void print_usage(char *msg, ...) __attribute__((noreturn));

/* Include file for additions: new protocols and new mappings. */
struct ipnat_proto
{
	struct ipnat_proto *next;

	/* Protocol number. */
	int protonum;

	/* Function which prints out usage message. */
	void (*help)(void);

	/* Function which parses command options; returns true if it
           ate an option */
	int (*parse)(int c, char **argv, struct ip_nat_rule_user *rule);

	/* Pointer to list of extra command-line options */
	struct option *extra_opts;
};

struct ipnat_mapping
{
	struct ipnat_mapping *next;

	/* Name of the mapping */
	const char *name;

	/* Bytes of options required. */
	size_t optlen;

	/* Function which prints out usage message. */
	void (*help)(void);

	/* Function which parses command options; returns true if it
           ate an option */
	int (*parse)(int c, char **argv, void *options, unsigned int *flags);

	/* Check flags. */
	void (*finalize)(const struct ip_nat_rule_user *rule,
			 void *options, unsigned int flags);

	/* Pointer to list of extra command-line options */
	struct option *extra_opts;
};

/* Your shared library should call one of these. */
extern void register_protocol(struct ipnat_proto *me);
extern void register_mapping(struct ipnat_mapping *me);
#endif /*_IPNATCTL_H*/
