#! /bin/sh

# Test the ordering of NAT rules.

# Need job control.
set -m

[ -n "$VERBOSE" ] && echo Stage 1: Setting up rules.
IP=100
for srcopt in "" "-s $TAP0NET.0/30" "-s $TAP0NET.2"
do
    for dstopt in "" "-d $TAP1NET.0/30" "-d $TAP1NET.2"
    do
	for protoopt in "" "-p 17" "-p 17 --sport 1" "-p 17 --sport 1 --dport 2"
	do
	    ../NAT/userspace/ipnatctl -I -n $srcopt $dstopt $protoopt -b source -t $TAP0NET.$IP || exit 1
	    IP=`expr $IP + 1`
	done
    done
done

# Receive 36 packets.
./rcv_ip 2 36 < /dev/tap1 > $TMPFILE &
sleep 1

[ -n "$VERBOSE" ] && echo Stage 2: Generating packets.

for srcopt in "$TAP0NET.99" "$TAP0NET.3" "$TAP0NET.2"
do
    for dstopt in "$TAP1NET.99" "$TAP1NET.3" "$TAP1NET.2"
    do
	for protoopt in "2" "17 5 5" "17 1 5" "17 1 2"
	do
	    ./gen_ip $srcopt $dstopt 100 $protoopt > /dev/tap0
	done
    done
done

[ -n "$VERBOSE" ] && echo Stage 3: Deleting rules.

IP=100
for srcopt in "" "-s $TAP0NET.0/30" "-s $TAP0NET.2"
do
    for dstopt in "" "-d $TAP1NET.0/30" "-d $TAP1NET.2"
    do
	for protoopt in "" "-p 17" "-p 17 --sport 1" "-p 17 --sport 1 --dport 2"
	do
	    ../NAT/userspace/ipnatctl -D -n $srcopt $dstopt $protoopt -b source -t $TAP0NET.$IP || exit 1
	    IP=`expr $IP + 1`
	done
    done
done

if wait %./rcv_ip; then :
else
    echo ./rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

[ -n "$VERBOSE" ] && echo Stage 4: Checking answers

IP=100
for srcopt in "$TAP0NET.99" "$TAP0NET.3" "$TAP0NET.2"
do
    for dstopt in "$TAP1NET.99" "$TAP1NET.3" "$TAP1NET.2"
    do
	for protoopt in "2" "17 5 5" "17 1 5" "17 1 2"
	do
	    read LINE
	    if [ "$LINE" != "$TAP0NET.$IP $dstopt 100 $protoopt" ]
	    then
		echo Unexpected answer \`"$LINE"\' vs \`"$TAP0NET.$IP $dstopt 100 $protoopt"\'
		exit 1
	    fi
	    IP=`expr $IP + 1`
	done
    done
done < $TMPFILE

exit 0
